/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.sync;

import static io.quarkus.scheduler.Scheduled.ConcurrentExecution.*;

import jakarta.enterprise.context.ApplicationScoped;
import jakarta.inject.Inject;

import de.ozgcloud.user.common.lock.LockService;
import io.quarkus.scheduler.Scheduled;
import lombok.extern.log4j.Log4j2;

@Log4j2
@ApplicationScoped
public class SyncScheduler {

	@Inject
	SyncService syncService;
	@Inject
	LockService syncLockService;

	@Scheduled(every = "{ozgcloud.usersync.period}", concurrentExecution = SKIP)
	public void start() {
		if (syncLockService.isNotLocked()) {
			var lock = syncLockService.lockCollection();

			sync(lock.getTimestamp());
		} else {
			LOG.debug("UserManager: Database is locked, no sync happend.");
			if (syncLockService.isLockOlderThanADay()) {
				LOG.warn("Database lock is older than a day, removing it now");
				syncLockService.unlock();
			}
		}
	}

	private void sync(long syncTimestamp) {
		try {
			syncService.sync(syncTimestamp);
		} catch (Exception e) {
			LOG.error("Error syncing UserManager database.", e);
		} finally {
			syncLockService.unlock();
		}
	}
}