/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.ArrayDeque;
import java.util.Deque;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Spy;

class RoleHierarchyTest {

	private final RoleHierarchy subRole = RoleHierarchy.builder().roleName(UserRole.VERWALTUNG_USER).build();
	@Spy
	private final RoleHierarchy root = RoleHierarchy.builder()
			.roleName(UserRole.VERWALTUNG_LOESCHEN)
			.subRole(subRole)
			.build();

	@Nested
	class TestGetFlattenedHierarchy {

		@Test
		void shouldCollectSubRoles() {
			root.getFlattenedHierarchy(UserRole.VERWALTUNG_LOESCHEN);

			verify(root).collectSubRoles(eq(root), eq(UserRole.VERWALTUNG_LOESCHEN), any(Deque.class));
		}
	}

	@Nested
	class TestCollectSubRoles {

		private static final String UNKNOWN_ROLE = "unknown";

		private final Deque<String> collectedSubRoles = new ArrayDeque<>();

		@Test
		void shouldAddLast() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_LOESCHEN, collectedSubRoles);

			assertThat(collectedSubRoles).containsExactly(UserRole.VERWALTUNG_LOESCHEN);
		}

		@Test
		void shouldReturnTrue() {
			var result = root.collectSubRoles(root, UserRole.VERWALTUNG_LOESCHEN, collectedSubRoles);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var result = root.collectSubRoles(root, UNKNOWN_ROLE, collectedSubRoles);

			assertThat(result).isFalse();
		}

		@Test
		void shouldCollectSubRoles() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_USER, collectedSubRoles);

			verify(root).collectSubRoles(subRole, UserRole.VERWALTUNG_USER, collectedSubRoles);
		}

		@Test
		void shouldReturnTrueOnExistingRole() {
			var result = root.collectSubRoles(root, UserRole.VERWALTUNG_USER, collectedSubRoles);

			assertThat(result).isTrue();
		}

		@Test
		void shouldAddLastSubRole() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_USER, collectedSubRoles);

			assertThat(collectedSubRoles).containsExactly(UserRole.VERWALTUNG_LOESCHEN, UserRole.VERWALTUNG_USER);
		}

		@Test
		void shouldRemoveLast() {
			root.collectSubRoles(root, UserRole.VERWALTUNG_LOESCHEN, collectedSubRoles);

			assertThat(collectedSubRoles).containsExactly(UserRole.VERWALTUNG_LOESCHEN);
		}

	}

}